<?php

namespace Odhen\API\Service;


class Preco {

    protected $entityManager;
    protected $util;
    protected $date;

    public function __construct(
        \Doctrine\ORM\EntityManager $entityManager,
        \Odhen\API\Util\Util $util,
        \Odhen\API\Util\Date $date) {

        $this->entityManager = $entityManager;
        $this->util = $util;
        $this->date = $date;
    }

    public function buscaPreco($CDFILIAL, $CDCLIENTE, $CDPRODUTO, $CDLOJA, $CDCONSUMIDOR) {
        $tabelaEscolhida = self::buscaTabelaVigente($CDFILIAL, $CDLOJA, $CDCLIENTE);
        if ($tabelaEscolhida['error'] == false) {
            $retorno = self::buscaPrecoProduto(
                $tabelaEscolhida["CDFILIAL"],
                $tabelaEscolhida["CDTABEPREC"],
                $CDPRODUTO,
                $tabelaEscolhida['DTINIVGPREC'],
                $CDCLIENTE,
                $CDCONSUMIDOR
            );
        }
        else {
            $retorno = $tabelaEscolhida;
        }
        return $retorno;
    }

    public function buscaTabelaVigente($CDFILIAL, $CDLOJA, $CDCLIENTE) {
        // busca tabela de preço por CLIENTEFILIAL
        $params = array(
            "CDFILIAL" => $CDFILIAL,
            "CDCLIENTE" => $CDCLIENTE
        );

        $tabelaClienteFilial = $this->entityManager->getConnection()->fetchAssoc(self::CLIENTE_FILIAL, $params);
        if ($tabelaClienteFilial["CDTABEPREC"]) {
            // valida a vigência
            $DTINIVGPREC = self::validaVigencia($tabelaClienteFilial['CDCFILTABPRE'], $tabelaClienteFilial["CDTABEPREC"]);
            if ($DTINIVGPREC == false) {
                $retorno = array(
                    'error' => true,
                    'message' => 'Tabela de preços associada ao cliente/filial está inativa.'
                );
            } else {
                $retorno = array(
                	'error' => false,
                    'CDFILIAL' => $tabelaClienteFilial["CDCFILTABPRE"],
                    'CDTABEPREC' => $tabelaClienteFilial["CDTABEPREC"],
                    'DTINIVGPREC' => $DTINIVGPREC,
                    'error' => false
                );
            }
        } else {
            // busca a tabela de preço por CLIENTE
            $params = array(
                'CDCLIENTE' => $CDCLIENTE
            );
            $tabelaCliente = $this->entityManager->getConnection()->fetchAssoc(self::EXISTE_PRECO_CLIE, $params);
            if ($tabelaCliente["CDTABEPREC"]) {
                // valida a vigência
                $DTINIVGPREC = self::validaVigencia($tabelaCliente["CDFILTABPREC"], $tabelaCliente["CDTABEPREC"]);
                if ($DTINIVGPREC == false) {
                    $retorno = array(
                        'error' => true,
                        'message' => 'Tabela de preços associada ao cliente está inativa.'
                    );
                } else {
                    $retorno = array(
                        'error' => false,
                        'CDFILIAL' => $tabelaCliente["CDFILTABPREC"],
                        'CDTABEPREC' => $tabelaCliente["CDTABEPREC"],
                        'DTINIVGPREC' => $DTINIVGPREC
                    );
                }
            } else {
                // busca tabela de preco por LOJA
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDLOJA' => $CDLOJA
                );
                $tabelaLoja = $this->entityManager->getConnection()->fetchAssoc(self::TABELA_PRECO_LOJA, $params);
                if ($tabelaLoja["CDTABEPREC"]) {
                    // valida a vigência
                    $DTINIVGPREC = self::validaVigencia($CDFILIAL, $tabelaLoja["CDTABEPREC"]);
                    if ($DTINIVGPREC == false) {
                        $retorno = array(
                            'error' => true,
                            'message' => 'Tabela de preços associada à loja está inativa.'
                        );
                    } else {
                        $retorno = array(
                            'error' => false,
                            'CDFILIAL' => $CDFILIAL,
                            'CDTABEPREC' => $tabelaLoja["CDTABEPREC"],
                            'DTINIVGPREC' => $DTINIVGPREC
                        );
                    }
                } else {
                    // busca tabela de preço por FILIAL (padrão)
                    $params = array(
                        'CDFILIAL' => $CDFILIAL
                    );
                    $paravend = $this->entityManager->getConnection()->fetchAssoc(self::PARAVEND, $params);
                    // se não tiver tabela de preço parametrizada retorna código de erro
                    if ((bool)$paravend["CDTABEPREC"] == false) {
                        $retorno = array(
                            'error' => true,
                            'message' => 'Não há tabela de preços cadastrada.'
                        );
                    } else {
                        // valida a vigência
                        $DTINIVGPREC = self::validaVigencia($CDFILIAL, $paravend["CDTABEPREC"]);
                        if ($DTINIVGPREC == false) {
                            $retorno = array(
                                'error' => true,
                                'message' => 'Tabela de preços está inativa.'
                            );
                        } else {
                            $retorno = array(
                                'error' => false,
                                'CDFILIAL' => $CDFILIAL,
                                'CDTABEPREC' => $paravend["CDTABEPREC"],
                                'DTINIVGPREC' => $DTINIVGPREC
                            );
                        }
                    }
                }
            }
        }
        return $retorno;
    }

    public function validaVigencia($CDFILIAL, $CDTABEPREC) {
        // valida vigência
        $params = array(
            'CDFILIAL'   => $CDFILIAL,
            'CDTABEPREC' => $CDTABEPREC
        );
        if ($this->util->databaseIsOracle()) {
            $retorno = $this->entityManager->getConnection()->fetchAssoc(self::TABELA_VENDA_ORACLE, $params);
        } else {
            $retorno = $this->entityManager->getConnection()->fetchAssoc(self::TABELA_VENDA_MSDE, $params);
        }
        if ($retorno) {
            return $retorno["DTINIVGPREC"];
        } else {
            return false;
        }
    }

    public function buscaPrecoProduto($CDFILIAL, $CDTABEPREC, $CDPRODUTO, $DTINIVGPREC, $CDCLIENTE, $CDCONSUMIDOR){
        $retorno = array(
            'error' => false,
            'PRECOCLIE' => 0,
            'PRECOVAR' => 0,
            'PRECOSUGER' => 0,
            'DESC' => 0,
            'ACRE' => 0,
            'PRECO' => 0,
            'retirarProduto' => false
        );

        // formata a data de vigência
        if ($this->util->databaseIsOracle()){
            $DTINIVGPREC = $this->date->getDataDeString($DTINIVGPREC, "d-m-Y H:i:s")->format(\Odhen\API\Util\Date::FORMATO_BRASILEIRO);
        } else {
            $DTINIVGPREC = $this->date->getDataDeString($DTINIVGPREC, "Y-m-d H:i:s")->format(\Odhen\API\Util\Date::FORMATO_BRASILEIRO);
        }

        // busca dados do produto
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDTABEPREC' => $CDTABEPREC,
            'DTINIVGPREC' => $DTINIVGPREC,
            'CDPRODUTO' => $CDPRODUTO
        );
        if ($this->util->databaseIsOracle()) {
            $itemTabPreco = $this->entityManager->getConnection()->fetchAssoc(self::ITEM_PRECO_ORACLE, $params);
        } else {
            $itemTabPreco = $this->entityManager->getConnection()->fetchAssoc(self::ITEM_PRECO_MSDE, $params);
        }

        // se não tiver preço cadastrado retorna o código do erro
        if (empty($itemTabPreco)) {
            $retorno["error"] = true;
            $retorno["message"] = 'Preço não cadastrado para o produto ' . $CDPRODUTO . '.';
            $retorno['retirarProduto'] = true;
        // verifica se o preco está zerado
        } elseif ($itemTabPreco["PRECO"] + $itemTabPreco["PRECOCLIE"] === 0) {
            $retorno["error"] = true;
            $retorno["message"] = 'O preço do produto ' . $CDPRODUTO . ' está zerado.';
        } else {
            // caso o produto seja válido, busca preco do produto
            $result       = self::precoDia($CDFILIAL, $CDTABEPREC, $CDPRODUTO, $DTINIVGPREC, $CDCLIENTE, $CDCONSUMIDOR);

            $precoDia     = floatval($result["VRPRECODIA"]); //VRPRECODIA
            $perc         = $result["PERC"]; //IDPERVALORPR
            $descAcre     = $result["DESCACRE"]; //IDDESCACREPR
            $idVisuaCupom = $result["IDVISUACUPOM"]; //IDVISUACUPOM

            $retorno["PRECOVAR"]   = floatval($itemTabPreco["IDPRECVARIA"]);
            $retorno["PRECOCLIE"]  = floatval($itemTabPreco["PRECOCLIE"]);
            $retorno["PRECOSUGER"] = floatval($itemTabPreco["PRECOSUGER"]);

            // verifica se existe preco do dia
            if (empty($precoDia)) {
                $retorno["PRECO"] = floatval($itemTabPreco["PRECO"]);
            } else {
                // acrescenta ou desconta, o valor ou percentual, do preço cadastrado
                $retorno["PRECOVAR"]   = floatval($itemTabPreco["IDPRECVARIA"]);
                $retorno["PRECOCLIE"]  = floatval($itemTabPreco["PRECOCLIE"]);
                $retorno["PRECOSUGER"] = 0;

                // define se é valor (V) ou percentual (P)
                if ($perc == "V") {
                    // define se é desconto (D) ou acréscimo (A)
                    if ($descAcre == "A") {
                        // define se desconto aparece no cupom ou não
                        if ($idVisuaCupom == "N") {
                            $retorno["ACRE"] = 0;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"] + $precoDia, 2));
                        } else {
                            $retorno["ACRE"] = $precoDia;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"], 2));
                        }
                    } else {
                        // define se desconto aparece no cupom ou não
                        if ($idVisuaCupom == "N") {
                            $retorno["DESC"] = 0;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"] - $precoDia, 2));
                        } else {
                            $retorno["DESC"] = $precoDia;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"], 2));
                        }
                    }
                } else {
                    // define se é desconto (D) ou acréscimo (A)

                    if ($descAcre == "A") {
                        $acrescimoAux = floatval(round(($itemTabPreco["PRECO"] * $precoDia) / 100, 2));
                        // define se desconto aparece no cupom ou não
                        if ($idVisuaCupom == "N") {
                            $retorno["ACRE"] = 0;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"] + $acrescimoAux, 2));
                        } else {
                            $retorno["ACRE"] = $acrescimoAux;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"], 2));
                        }
                    } else {
                        $itemTabPreco["PRECO"] = floor(floatval($itemTabPreco["PRECO"]) * 100) / 100;
                        $descontoAux = floor(floatval(($itemTabPreco["PRECO"] * $precoDia) / 100) * 100) / 100;

                        // define se desconto aparece no cupom ou não
                        if ($idVisuaCupom == "N") {
                            $retorno["DESC"] = 0;
                            $retorno["PRECO"] = floor(($itemTabPreco["PRECO"] - $descontoAux) * 100) / 100;

                            if ($retorno["PRECO"] < 0.01) {
                                $retorno["PRECO"] = 0.01;
                            }
                        } else {
                            $retorno["DESC"] = $descontoAux;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"], 2));
                        }
                    }
                }
            }
        }

        return $retorno;
    }

    /**
     * retorno: [VRPRECODIA, PERC, DESCACRE,IDVISUACUPOM]
     */
    public function precoDia($CDFILIAL, $CDTABEPREC, $CDPRODUTO, $DTINIVGPREC, $CDCLIENTE, $CDCONSUMIDOR) {
        $dataAtual   = $this->date->getDataAtual();
        $diaSeguinte = $this->date->adicionaIntervalo($dataAtual);

        $params = array(
            'CDCLIENTE'    => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );

        // busca o tipo de consumidor do consumidor passado
        $retorno = $this->entityManager->getConnection()->fetchAssoc(self::TIPO_CONSUMIDOR, $params);
        if ($retorno) {
            $tipoConsumidor = $retorno["CDTIPOCONS"];
        } else {
            $tipoConsumidor = "T";
        }

        $params = array(
            'CDFILIAL'   => $CDFILIAL,
            'DTFERIFILI' => $dataAtual->format("d/m/Y")
        );

        // verifica se é feridado
        $retorno = $this->entityManager->getConnection()->fetchAssoc(self::FERIADO, $params);

        if ($retorno) {
            $dia = "F";
        } else {
            // verifica se o dia seguinte é feriado (significa que hoje é véspera de feriado)
            // comentado, Murillo do Madero falou que o desconto do happy hour aplica na véspera também

            /*
            $retorno = $this->entityManager->getConnection()->fetchAssoc (
                self::FERIADO, array($CDFILIAL, $diaSeguinte->format("d/m/Y"))
            );
            if ($retorno) {
                $dia = "V";
            } else {
            */
            // Expected: 1 (for Sunday) through 7 (for Saturday)
            // PHP returns: 1 (for Monday) through 7 (for Sunday)
            // So, this method changes the index for each day, changing from the format of return
            // used by PHP to Delphi format.
            $dia = $dataAtual->format("N") % 7 + 1;
            //}
        }

        // 1 domingo
        // 2 segunda
        // 3 terca
        // 4 quarta
        // 5 quinta
        // 6 sexta
        // 7 sabado
        // F - feriado
        // V - dia antes do feriado (véspera)

        $hora = $dataAtual->format("H:i:s");
        $hora = str_replace(":", "", $hora);
        $hora = substr($hora, 0, 4);
        // $hora = '1930';

        $params = array(
            'CDFILIAL'     => $CDFILIAL,
            'CDTABEPREC'   => $CDTABEPREC,
            'DTINIVGPREC'  => $DTINIVGPREC,
            'CDPRODUTO'    => $CDPRODUTO,
            'CDPRPAITABPR' => $CDPRODUTO,
            'NRDIASEMANPR' => (string) $dia,
            'CDTIPOCONSPD' => $tipoConsumidor,
            'HORA'         => $hora
        );

        if ($this->util->databaseIsOracle()) {
            $itemPrecoDia = $this->entityManager->getConnection()->fetchAssoc(self::ITEM_PRECO_DIA_ORACLE, $params);
        } else {
            $itemPrecoDia = $this->entityManager->getConnection()->fetchAssoc(self::ITEM_PRECO_DIA_MSDE, $params);
        }

        if ($itemPrecoDia) {
            $retorno["PERC"]         = $itemPrecoDia["IDPERVALORPR"];
            $retorno["DESCACRE"]     = $itemPrecoDia["IDDESCACREPR"];
            $retorno["IDVISUACUPOM"] = $itemPrecoDia["IDVISUACUPOM"];
            $retorno["VRPRECODIA"]   = $itemPrecoDia["VRPRECODIA"];
        } else {
            $params = array(
                'CDFILIAL'     => $CDFILIAL,
                'CDTABEPREC'   => $CDTABEPREC,
                'DTINIVGPREC'  => $DTINIVGPREC,
                'CDPRODUTO'    => $CDPRODUTO,
                'CDPRPAITABPR' => $CDPRODUTO,
                'NRDIASEMANPR' => "T",
                'CDTIPOCONSPD' => $tipoConsumidor,
                'HORA'         => $hora
            );

            if ($this->util->databaseIsOracle()) {
                $itemPrecoDia = $this->entityManager->getConnection()->fetchAssoc(self::ITEM_PRECO_DIA_ORACLE, $params);
            } else {
                $itemPrecoDia = $this->entityManager->getConnection()->fetchAssoc(self::ITEM_PRECO_DIA_MSDE, $params);
            }

            if ($itemPrecoDia) {
                $retorno["PERC"]         = $itemPrecoDia["IDPERVALORPR"];
                $retorno["DESCACRE"]     = $itemPrecoDia["IDDESCACREPR"];
                $retorno["IDVISUACUPOM"] = $itemPrecoDia["IDVISUACUPOM"];
                $retorno["VRPRECODIA"]   = $itemPrecoDia["VRPRECODIA"];
            } else {
                // Expected: 1 (for Sunday) through 7 (for Saturday)
                // PHP returns: 1 (for Monday) through 7 (for Sunday)
                // So, this method changes the index for each day, changing from the forat of return
                // used by PHP to Delphi format.
                $dia = (((int)$dataAtual->format("N")) % 7) + 1;

                $params = array(
                    'CDFILIAL'     => $CDFILIAL,
                    'CDTABEPREC'   => $CDTABEPREC,
                    'DTINIVGPREC'  => $DTINIVGPREC,
                    'CDPRODUTO'    => $CDPRODUTO,
                    'CDPRPAITABPR' => $CDPRODUTO,
                    'NRDIASEMANPR' => (string) $dia,
                    'CDTIPOCONSPD' => $tipoConsumidor,
                    'HORA'         => $hora
                );

                if ($this->util->databaseIsOracle()) {
                    $itemPrecoDia = $this->entityManager->getConnection()->fetchAssoc(self::ITEM_PRECO_DIA_ORACLE, $params);
                } else {
                    $itemPrecoDia = $this->entityManager->getConnection()->fetchAssoc(self::ITEM_PRECO_DIA_MSDE, $params);
                }
                if ($itemPrecoDia) {
                    $retorno["PERC"]         = $itemPrecoDia["IDPERVALORPR"];
                    $retorno["DESCACRE"]     = $itemPrecoDia["IDDESCACREPR"];
                    $retorno["IDVISUACUPOM"] = $itemPrecoDia["IDVISUACUPOM"];
                    $retorno["VRPRECODIA"]   = $itemPrecoDia["VRPRECODIA"];
                } else {
                    $retorno = null;
                }
            }
        }

        return $retorno;
    }

    public function getSmartPromoPrice($CDPRODPROMOCAO, $CDGRUPROMOC, $CDPRODUTO, $CDFILIAL, $CDTABEPREC, $DTINIVGPREC){
        $DTINIVGPREC = $this->date->getDataDeString($DTINIVGPREC, "d-m-Y H:i:s")->format(\Odhen\API\Util\Date::FORMATO_BRASILEIRO);

        $params = array(
            ':CDPRODPROMOCAO' => $CDPRODPROMOCAO,
            ':CDGRUPROMOC'    => $CDGRUPROMOC,
            ':CDPRODUTO'      => $CDPRODUTO,
            ':CDFILIAL'       => $CDFILIAL,
            ':CDTABEPREC'     => $CDTABEPREC,
            ':DTINIVGPREC'    => $DTINIVGPREC
        );
        $result = $this->entityManager->getConnection()->fetchAll(self::GET_SMARTPROMO_PRICE, $params);
        return !empty($result) ? $result[0] : $result;

    }

    const GET_SMARTPROMO_PRICE = "
      SELECT NVL(IT.VRPRECITEM, 0) AS PRECO,
             DPP.IDPERVALORDES AS TIPO,
             DPP.VRDESPRODPROMOC AS VRDESC,
             0 AS ACRE
        FROM PRODGRUPROMOC PGP
        JOIN GRUPROMOCPROD GPP
          ON (PGP.CDGRUPROMOC = GPP.CDGRUPROMOC)
        JOIN PRODUTO P
          ON (PGP.CDPRODUTO = P.CDPRODUTO)
        JOIN ITEMPRECO IT
          ON (PGP.CDPRODUTO = IT.CDPRODUTO) AND
             (IT.CDPRODUTO = P.CDPRODUTO)
        LEFT JOIN DESGRUPROMOCPR DPP
          ON (PGP.CDGRUPROMOC = DPP.CDGRUPROMOC) AND
             (PGP.CDPRODUTO = DPP.CDPRODUTO) AND
             (GPP.CDPRODPROMOCAO = DPP.CDPRODPROMOCAO)
       WHERE (GPP.CDPRODPROMOCAO = :CDPRODPROMOCAO) AND
             (PGP.CDGRUPROMOC = :CDGRUPROMOC) AND
             (IT.CDPRODUTO = :CDPRODUTO) AND
             (IT.CDFILIAL = :CDFILIAL) AND
             (IT.CDTABEPREC = :CDTABEPREC) AND
             (IT.DTINIVGPREC = :DTINIVGPREC)
    ";

    const CLIENTE_FILIAL = "
        SELECT CDTABEPREC, CDCFILTABPRE
          FROM CLIENFILIAL
          WHERE CDFILIAL = :CDFILIAL
          AND CDCLIENTE = :CDCLIENTE";

    const TABELA_VENDA_MSDE = "
        SELECT CONVERT(VARCHAR,DTINIVGPREC,120) AS DTINIVGPREC
          FROM TABEVEND
         WHERE CDFILIAL   = :CDFILIAL
           AND CDTABEPREC = :CDTABEPREC
           AND (CONVERT(VARCHAR, GETDATE(), 103) BETWEEN DTINIVGPREC AND DTFINVGPREC)";

    const TABELA_VENDA_ORACLE = "
        SELECT TO_CHAR(A.DTINIVGPREC, 'DD-MM-YYYY HH:MI:SS') AS DTINIVGPREC FROM (
            SELECT TO_TIMESTAMP(TO_CHAR(DTINIVGPREC, 'DD-MM-YYYY HH:MI:SS'), 'DD-MM-YYYY HH:MI:SS') AS DTINIVGPREC
              FROM TABEVEND
             WHERE CDFILIAL   = :CDFILIAL
               AND CDTABEPREC = :CDTABEPREC
               AND SYSDATE BETWEEN DTINIVGPREC AND DTFINVGPREC
        ) A";

    const ITEM_PRECO_MSDE = "
        SELECT ISNULL(I.VRPRECITEM, 0) AS PRECO,
               ISNULL(I.VRPRECITEMCL, 0) AS PRECOCLIE,
               ISNULL(I.VRPRESUGITEM, 0) AS PRECOSUGER,
               I.IDPRECVARIA, P.NMPRODUTO
          FROM ITEMPRECO I JOIN PRODUTO P
                             ON I.CDPRODUTO = P.CDPRODUTO
         WHERE I.CDFILIAL = :CDFILIAL
           AND I.CDTABEPREC = :CDTABEPREC
           AND I.DTINIVGPREC = :DTINIVGPREC
           AND I.CDPRODUTO = :CDPRODUTO
    ";

    const ITEM_PRECO_ORACLE = "
        SELECT NVL(I.VRPRECITEM, 0) AS PRECO,
               NVL(I.VRPRECITEMCL, 0) AS PRECOCLIE,
               NVL(I.VRPRESUGITEM, 0) AS PRECOSUGER,
               I.IDPRECVARIA, P.NMPRODUTO
          FROM ITEMPRECO I JOIN PRODUTO P
                             ON I.CDPRODUTO = P.CDPRODUTO
         WHERE I.CDFILIAL = :CDFILIAL
           AND I.CDTABEPREC = :CDTABEPREC
           AND DTINIVGPREC = TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY HH:MI:SS')
           AND I.CDPRODUTO = :CDPRODUTO
    ";

    const TIPO_CONSUMIDOR = "
        SELECT CDTIPOCONS
          FROM CONSUMIDOR
         WHERE CDCLIENTE = :CDCLIENTE
           AND CDCONSUMIDOR = :CDCONSUMIDOR
    ";

    const FERIADO = "
        SELECT DTFERIFILI
          FROM FERIFILI
         WHERE CDFILIAL = :CDFILIAL
           AND DTFERIFILI = :DTFERIFILI
    ";

    const ITEM_PRECO_DIA_MSDE = "
        SELECT VRPRECODIA, IDPERVALORPR, IDDESCACREPR, IDVISUACUPOM, CDTIPOCONSPD
          FROM ITEMPRECODIA
          WHERE CDFILIAL       = :CDFILIAL
            AND CDTABEPREC     = :CDTABEPREC
            AND DTINIVGPREC    = :DTINIVGPREC
            AND CDPRODUTO      = :CDPRODUTO
            AND CDPRPAITABPR   = :CDPRPAITABPR
            AND NRDIASEMANPR   = :NRDIASEMANPR
            AND ((CDTIPOCONSPD = :CDTIPOCONSPD) OR (CDTIPOCONSPD = 'T'))
            AND :HORA BETWEEN HRINIPRECDIA AND HRFINPRECDIA
            AND (CONVERT(VARCHAR, GETDATE(), 103) BETWEEN DTINIVALPREC AND DTFINVALPREC)";

    const ITEM_PRECO_DIA_ORACLE = "
        SELECT VRPRECODIA, IDPERVALORPR, IDDESCACREPR, IDVISUACUPOM, CDTIPOCONSPD
          FROM ITEMPRECODIA
          WHERE CDFILIAL       = :CDFILIAL
            AND CDTABEPREC     = :CDTABEPREC
            AND DTINIVGPREC    = TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY HH:MI:SS')
            AND CDPRODUTO      = :CDPRODUTO
            AND CDPRPAITABPR   = :CDPRPAITABPR
            AND NRDIASEMANPR   = :NRDIASEMANPR
            AND ((CDTIPOCONSPD = :CDTIPOCONSPD) OR (CDTIPOCONSPD = 'T'))
            AND TO_CHAR(SYSDATE,'HH24')||'00' BETWEEN HRINIPRECDIA AND HRFINPRECDIA";

    const EXISTE_PRECO_CLIE = "
        SELECT CDTABEPREC, CDFILTABPREC
          FROM CLIENTE
          WHERE (CDCLIENTE = :CDCLIENTE)";

    const TABELA_PRECO_LOJA = "
        SELECT CDTABEPREC
          FROM LOJA
          WHERE (CDFILIAL = :CDFILIAL)
            AND (CDLOJA   = :CDLOJA)";

    const PARAVEND = "
        SELECT CDTABEPREC, IDPRECDIFCOM
          FROM PARAVEND
          WHERE (CDFILIAL = :CDFILIAL)";
}