<?php

namespace Zeedhi\PaymentPlatform\examples;

$ds = DIRECTORY_SEPARATOR;

require_once __DIR__.$ds.'..'.$ds.'..'.$ds.'vendor'.$ds.'autoload.php';

use Zeedhi\PaymentPlatform\Implementations\MundiPagg\PaymentImpl;
$paymentPlatform = PaymentImpl::factory("sk_test_kVe4L0yCEcEW4Nb9");

use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyPlatform;
use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction;

use Zeedhi\PaymentPlatform\Address;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\CreditCard;

class Order {

    private $paymentPlatform;

    public function __construct(InstantBuyPlatform $paymentPlatform) {
        $this->paymentPlatform = $paymentPlatform;
    }
    
    public function createInstantBuy($creditCard, $customer){
        return $this->paymentPlatform->createToken($creditCard, $customer);
    }

    public function createOrder($value, $transactionId, $instantBuy, Customer $customer){
        $transaction = new InstantBuyTransaction($value, $transactionId, $instantBuy, $customer);
        return $this->paymentPlatform->instantBuyTransaction($transaction);
    }
}

/**
 * InstantBuyTransaction.
 * 
 * É obrigatório pelo menos 'name' em Customer.
 * Caso o campo 'type' seja setado é necessário informar o tipo de pessoa. Valores disponíveis: Customer::INDIVIDUAL or Customer::COMPANY.
 * Para a MundiPagg o campo 'state' deve conter no máximo 2 caracteres.
 * Caso não exista Customer criado basta setar os valores como o exemplo abaixo.
 * Para ter acesso ao CustomerId criado consulte a transação na class Consult.
 * Se já existe Customer criado basta setar o id($customer->setId('id'))
 * 
 */
$customer = new Customer();
$customer->setName("Stephen Vincent Strange");
$customer->setEmail("dr-stranger@avengers.com");
$customer->setDocument("666999666");
$customer->setType(Customer::INDIVIDUAL);
$customer->setPhone("999645466");
$street = "Rua Napolitano";
$number = "456";
$complement = "Casa";
$neighborhood = "Estadual";
$city = "Rio Barbosa";
$state = "PI";
$zipCode = "123123123";
$country = "BR";
$address = new Address($street, $number, $complement, $neighborhood, $city, $state, $zipCode, $country);
$customer->setAddress($address);

$holderName = "Stephen Vincent Strange";
$numbers = "4000000000000044";
$expDateMonth = "07";
$expDateYear = "17";
$cvv = "123";
$flag = "Visa";
$creditCard = new CreditCard($holderName, $numbers, $expDateMonth, $expDateYear, $cvv, $flag);

$order = new Order($paymentPlatform);
$instantBuyAndCostumerId = $order->createInstantBuy($creditCard, $customer);
var_dump($instantBuyAndCostumerId);

$value = "1500.00";
$transactionId = "0666";

$customer = new Customer();
$customer->setId($instantBuyAndCostumerId["CustomerId"]);

$transactionKey = $order->createOrder($value, $transactionId, $instantBuyAndCostumerId["InstantBuyKey"], $customer);
var_dump($transactionKey);

