ZeedhiDirectives.directive('zhMasonryGrid', function($timeout, templateManager, ScreenService) {
	
	function _buildMatrix(arr, numRows, numCols, initial) {
		arr = arr || [];
		for (var y = 0; y < numRows; y++) {
			var cols = [];
			for (var x = 0; x < numCols; x++) {
				cols.push(initial);
			}
			arr.push(cols);
		}
		return arr;
	}

	function _generatePositionOfNextFreeSpace(matrix, numCols, width, height, buildOrder) {
		var isThereFreeSpace = false;
		var pos = {};
		var x, y, j, k;
		var rows = matrix.length;

		/**
		 * Calculates an absolute position when find a free space.
		 **/
		function _updateElementPositionIfFreeSpace() {
			if (matrix[y][x] === 0) { // empty space
				var allGood = true;
				if (x + width > numCols || y + height > rows) {
					allGood = false;
				}
				if (allGood) {
					for (j = 0; j < width; j++) {
						for (k = 0; k < height; k++) {
							if (matrix[y + k][x + j] !== 0) {
								allGood = false;
								return false;
							}
						}
						if (!allGood) {
							return false;
						}
					}
				}
				if (allGood) {
					isThereFreeSpace = true;
					pos.top = y;
					pos.left = x;
					for (j = 0; j < width; j++) {
						for (k = 0; k < height; k++) {
							matrix[y + k][x + j] = 1;
						}
					}
					return false;
				}
			}

			return true;
		}
		
		var firstOrderDirection = rows;
		var secondOrderDirection = numCols;
		if(buildOrder === 'cols'){
			firstOrderDirection = numCols;
			secondOrderDirection = rows;
		}
		
		for(y = 0; y < firstOrderDirection; y++){
			for(x = 0; x < secondOrderDirection; x++){
				if(!_updateElementPositionIfFreeSpace()) break; // free space not found on "secondOrderDirection"
			}
			if(isThereFreeSpace){ // free space found
				break;
			}
		}

		if (isThereFreeSpace) {
			return pos;
		} else {
			_buildMatrix(matrix, height, numCols, 0);
			return _generatePositionOfNextFreeSpace(matrix, numCols, width, height, buildOrder);
		}
	}

	var masonryGrids = [];
	$(window).on('resize.masonry', function() {
		masonryGrids.forEach(function(grid) {
			if (grid.is(':visible') && grid.rendered) {
				var newCols = grid.getColumns();
				if (newCols != grid.columns) {
					grid.columns = newCols;
					grid.render();
				} else {
					grid.positionAllBlocksOnDashboard();
				}
			}
		});
	});

	return {
		restrict: 'A',
		link: function($scope, element, $attrs) {

			element.columns = 0;
			element.lastRow = 0;
			element.gridItems = [];
			element.fillers = [];
			element.rendered = false;

			var DIVISION_FACTOR = 4;
			var buildOrder = $attrs.zhMasonryGridOrder || 'rows';
			var container = element;
			
			element.render = function(items) {
				var element = this;
				var container = templateManager.container;
				
				element.gridWidgets = _getOnlyVisibleWidgets(items || element.gridWidgets || _getDashboardWidgets());
				element.columns = element.getColumns();

				var widgetMatrix = _buildMatrix(null, 1, element.columns, 0);
				element.lastRow = 0;
				
				element.gridWidgets.forEach(function(widget){
					widget.computedDimensions = _getItemDimensions(element, widget);
					var position = _generatePositionOfNextFreeSpace(widgetMatrix, element.columns, Math.min(widget.computedDimensions.width, element.columns), widget.computedDimensions.height, buildOrder);
					widget.pos = position;

					if(position.top + widget.computedDimensions.height > element.lastRow){
						element.lastRow = position.top + widget.computedDimensions.height;
					}
					
				});

				if (hasAttr(element, 'zh-masonry-fillers')) {
					element.addFillers(widgetMatrix, element.lastRow);
				}
				
				element.positionAllBlocksOnDashboard();
			};
			this.render = element.render;
			
			var _isThereAnyWidgetWithHeightPercent = function(){
				var widgets = _getDashboardWidgets();
				return widgets.some(function(widget){
					return _isValuePercentage(widget.height);
				});
			};
			
			var _getOnlyVisibleWidgets = function(widgets){
				return widgets.filter(function(item){
					return item.isVisible && item.aclIsVisible && (item._searchOn || Util.isUndefined(item._searchOn));
				});	
			};
			
			var _getDashboardWidgets = function(){
				return _isDashboardContainer(container) ? container.widgets : $scope.widget.widgets;	
			};
			
			var _isDashboardContainer = function(container){
				return container.template && container.template.indexOf('container/dashboard.html') !== -1;
			};
			
			function hasAttr(element, attr) {
				return (typeof element.attr(attr) !== "undefined" && element.attr(attr) !== false) || (typeof element.attr('data-' + attr) !== "undefined" &&
					element.attr('data-' + attr) !== false);
			}
			
			var _getItemDimensions = function(container, item) {
				item.dimensions = item.dimensions || [];
				var convertedColumns = container.columns / DIVISION_FACTOR; // (52 / 4) = 13
				var dimensions;
				
				item.dimensions.forEach(function(dimension){
					if( !_isItemColumnsLessThanMinimum(convertedColumns, dimension.minCols) &&
					    !_isItemColumnsMoreThanMaximum(convertedColumns, dimension.maxCols) )
					{
						dimensions = { width: dimension.width, height: dimension.height };
					}
				});
				
				if(Util.isUndefined(dimensions)){
					dimensions = { width: item.width || 1, height: item.height || 1 };	
				}
				
				if(_isValuePercentage(dimensions.width)) {
					dimensions.width = _convertItemWidthPercentage(dimensions.width, container);
				} else {
					dimensions.width *= DIVISION_FACTOR;
				}
				
				if(_isValuePercentage(dimensions.height)){
					dimensions.height = _convertItemHeightPercentage(dimensions.height, container);
				} else {
					dimensions.height *= DIVISION_FACTOR;
				}

				return dimensions;
			};
			
			var _isItemColumnsLessThanMinimum = function(columns, minColumns){
				return !Util.isUndefined(minColumns) && columns <= minColumns;
			};
			
			var _isItemColumnsMoreThanMaximum = function(columns, maxColumns){
				return !Util.isUndefined(maxColumns) && columns >= maxColumns;
			};
			
			var _convertItemWidthPercentage = function(itemWidth, container){
				itemWidth = itemWidth.toString().replace('%', '');
				return (Math.ceil(itemWidth) * container.columns) / 100;
			};
			
			var _convertItemHeightPercentage = function(itemHeight, container){
				itemHeight = itemHeight.toString().replace('%', '');
				return (Math.ceil(itemHeight * _getMaxVisibleRows())) / 100; // 'columns' temporally, should be 'rows'
			};
			
			var _isValuePercentage = function(value){
				return value.toString().slice(-1) === '%';
			};

			element.getColumns = function() {
				var container = this;
				return Math.max(2, Math.floor(container.width() / 100)) * DIVISION_FACTOR;
			};
			
			var _getMaxVisibleRows = function(){
				return Math.floor(container.height() / 100) * DIVISION_FACTOR;
			};

			var _getVerticalBlankSpaceInPixels = function(){
				container.css('height', container.lastRow * unitSize + padding * 2);
				
			};

			element.addFillers = function(matrix, rows) {
				var container = this;
				container.fillers = [];
				container.find('.filler').remove();
				for (var y = 0; y < rows; y++) {
					for (var x = 0; x < container.columns; x++) {
						if (matrix[y][x] === 0) { // if blank space
							var filler = $('<div class="masonry-item filler"></div>');
							container.append(filler);
							container.fillers.push({
								'element': filler,
								'pos': {
									'top': y,
									'left': x
								}
							});
						}
					}
				}
			};

			element.positionAllBlocksOnDashboard = function() {
				var container = this;
				var unitSize = container.width() / container.columns; // ex.: 26
				var padding = parseInt(container.css('padding-top'));
				container.css('height', container.lastRow * unitSize + padding * 2);
				
				$timeout(function() {
					_positionWidgetsElements();
					_positionFillerElements();
					$timeout(function() {
						$scope.$broadcast("zh-masonry-grid-finished", container);
					}, 500); // must wait for the css animation to finish
				});
			};

			var _positionWidgetsElements = function(){
				var unitSize = container.width() / container.columns;
				var padding = parseInt(container.css('padding-top'));
				
				container.gridWidgets.forEach(function(widget){
					var domElement = container.find('#masonry-item-' + widget.id);
					var position = widget.pos;
					domElement.addClass('masonry-item');
					
					if(widget.color){
						if(widget.color[0] === '#'){
							domElement.css('background-color', widget.color);
						} else {
							domElement.addClass('color' + (widget.color ? widget.color : 1));
						}
					}
					
					domElement.css('left', position.left * unitSize + padding);
					domElement.css('top', position.top * unitSize + padding);
					domElement.css('width', Math.min(widget.computedDimensions.width, container.columns) * unitSize);
					domElement.css('height', widget.computedDimensions.height * unitSize);
					
					// Border
					if(widget.divider){
						_createWidgetDivider(domElement, widget);
					}
					
					domElement.trigger('resize.item');
					
					if(widget.redraw){
						var redrawFunction = widget.redraw;
						$timeout(redrawFunction, 500);
					}
				});
			};

			var _createWidgetDivider = function(element, widget){
				element.css('border', ''); // removes default white border
				var borders = widget.divider.split("|");
				borders.forEach(function(border){
					_createDividerLine(element, border);
					_createDividerSpacing(element, border);
				});
			};
			
			var _createDividerLine = function(element, border){
				element.css('border-' + border, '2px dotted #e3e3e3');	
			};
			
			var _createDividerSpacing = function(element, border){
				var zhScrollerElement = element.find("[zh-scroller]");
				if(zhScrollerElement.length === 0){
					zhScrollerElement = element.find("[data-zh-scroller]");
				}
				zhScrollerElement.css('margin-' + border, '3px');
			};

			var _positionFillerElements = function _positionFillerElements() {
				var unitSize = container.width() / container.columns;
				var padding = parseInt(container.css('padding-top'));
				
				container.fillers.forEach(function(filler){
					var fillerDomElement = filler.element;
					var position = filler.pos;
					
					fillerDomElement.css('left', position.left * unitSize + padding);
					fillerDomElement.css('top', position.top * unitSize + padding);
					fillerDomElement.css('width', unitSize);
					fillerDomElement.css('height', unitSize);
				});
			};

			$scope.widget.searchCriteria = {};
			$scope.search = function(item) {
				var hasSearchCriteria = $scope.widget.searchCriteria && Util.objectHasAnyValue($scope.widget.searchCriteria);
				item._searchOn = !hasSearchCriteria || ScreenService.i18n(item.label).indexOfLatin($scope.widget.searchCriteria._ALL) !== -1;
				return item.isVisible && item.aclIsVisible && item._searchOn;
			};

			var _updateSearch = Util.buildDebounceMethod(function() {
				$scope.grid.render();
			});

			$scope.$watch(function() { return $scope.widget.searchCriteria; }, function(newValue, oldValue) {
				if (!angular.equals(newValue, oldValue)) {
					$timeout(_updateSearch, newValue ? 500 : 0);
				}
			}, true);

			$scope.initializeGrid = function(items) {
				$scope.grid = element;
				$scope.grid.rendered = false;

				$timeout(function() {
					$scope.grid.render(items);
					$scope.grid.rendered = true;
					masonryGrids.push($scope.grid);
				});
			};

			if (templateManager.container.onRender){
				templateManager.container.onRender(function(){
					element.render();
				});
			}
		}
	};
});