describe('ContainerLoader', function() {

	var zhPromise = new ZHPromise(Q);
	var screenService = { i18n: function(text){ return text; } };
	var templateManager = {
		project: {},
		projectPromise: zhPromise.when(),
		containers: {},
		getMappedPackages: function() {
			return {
				default: ["./"]
			};
		}
	};
	var thenable = function(data) {
		return {
			then : function(fn){
				var dataReturned;
				if(fn){
					dataReturned = fn(data);
				}
				return thenable(dataReturned || data);
			}
		};
	};

	var defaultContainer;
	var defaultContainerPreference;

	var requestDataSourceEngine = {
		save: function(){
			return thenable();
		}
	};
	var localStorageEngine = {
		save: function(){
			return thenable();
		},
		getDataSource: function(){
			return thenable([defaultContainerPreference]);
		},
		sync: function(){
			return thenable([defaultContainerPreference]);
		}
	};
	var restEngine = new RestEngine(console, function(p) {
		lastParams = p;
		return {
			success: function(fn) {
				if(fn) {
					fn(defaultContainer);
				}
				return thenable();
			}
		};
	}, templateManager, {}, new ZhFeatures(console), zhPromise, {
		onRequestStart: function() {},
		onRequestMetaDataStart: function () {}
	}, {});

	var repositoryFactory = {
		factory: function(){
			return {
				find: function(){
					return zhPromise.when([defaultContainerPreference]);
				},
				save: function(){
					return zhPromise.when();
				}
			};
		}
	};

	var zhUtil = {
		registerDataSource : function(name){
			repoTest = name;
	}};

	var fallbackStorageStrategy = {};
	var synchronizableStorageEngineFactory = new SynchronizableStorageEngineFactory(zhPromise);
	var preferenceManager = new PreferenceManager(synchronizableStorageEngineFactory, fallbackStorageStrategy, localStorageEngine, requestDataSourceEngine, new QueryBuilder(), screenService, zhPromise, zhUtil, repositoryFactory);
	var containerRepository = new ContainerRepository(preferenceManager);
	var containerMerger = new ContainerMerger();
	var containerDecoratorPreference = new ContainerDecoratorPreference(containerRepository, screenService, zhPromise, containerMerger);
	var containerDecoratorProvider = new ContainerDecoratorProvider(containerDecoratorPreference);
	var jsonContainerLoader = new JSONContainerLoader(restEngine, templateManager, zhPromise, screenService);
	var containerLoader;

	beforeEach(function(){
		defaultContainer = {
			"name": "login",
			"showFooter": true,
			"showMenu": true,
			"widgets": [
				{
					"id": 1,
					"name": "loginWidget",
					"template": "widget/list.html",
					"actions": [
						{
							"id": 2,
							"name": "TEST",
							"showAsAction": "always",
							"events": [
								{
									"id": 3,
									"name": "ActionEvent",
									"code": ";"
								}
							]
						}
					],
					"events": [
						{
							"id": 4,
							"name": "WidgetOnEnter",
							"code": ";"
						}
					],
					"fields": [
						{
							"id": 5,
							"name": "TEST",
							"label": "Default label"
						}
					],
					"styles": [
						{
							"id": 6,
							"condition": "row.id % 2 == 0",
							"cssClass": "fuck"
						}
					],
					"fieldGroups": [
						{
							"id": 7,
							"name": "ORG",
							"label": "Dados da organização",
							"collapsible": true,
							"opened": true
						}
					],
					"widgets": [
						{
							"id": 100,
							"name": "secondWidget",
							"label": "Second widget",
							"events": [
								{
									"id": 102,
									"name": "WidgetOnEnter",
									"code": ";"
								}
							]
						}
					]
				}
			]
		};

		defaultContainerPreference = {
			"name": "login",
			"showFooter": false,
			"widgets": [
				{
					"id": 1,
					"name": "loginWidget",
					"template": "widget/form.html",
					"actions": [
						{
							"id": 2,
							"name": "TEST",
							"showAsAction": "never",
							"events": [
								{
									"id": 3,
									"name": "ActionEvent",
									"code": "args.owner.widget.newRow();"
								}
							]
						}
					],
					"events": [
						{
							"id": 4,
							"name": "WidgetOnEnter",
							"code": "args.owner.newRow();"
						}
					],
					"fields": [
						{
							"id": 5,
							"name": "TEST",
							"label": "Preference label"
						},{
							"id": 20,
							"name": "NEW_FIELD",
							"label": "New field"
						}
					],
					"styles" : [
						{
							"id": 6,
							"condition" : "row.ID % 2 === 0",
							"cssClass" : "zh-inactive"
						}
					],
					"fieldGroups": [
						{
							"id": 7,
							"name": "ORG changed",
							"label": "Dados da organização",
							"collapsible": true,
							"opened": false
						}
					],
					"widgets": [
						{
							"id": 1000,
							"name": "thirdWidget",
							"label": "3"
						},
						{
							"id": 100,
							"name": "secondWidget",
							"label": "2",
							"fields": [
								{
									"id": 101,
									"name": "second TEST",
									"label": "Second widget field label"
								}
							],
							"events": [
								{
									"id": 102,
									"name": "WidgetOnEnter",
									"code": "args.owner.newRow();"
								}
							]
						}
					]
				}
			]
		};
		containerLoader = new ContainerLoader(jsonContainerLoader, containerDecoratorProvider, zhPromise, screenService);
	});

	it("Method 'setConcreteContainerLoader': Should change the concreteLoader if is a valid loader", function(){
		var loader = { loadContainer: function() {} };
		expect(function(){
			containerLoader.setConcreteContainerLoader(loader);
		}).not.toThrow();
	});

	it("Method 'setConcreteContainerLoader': Should sent an exception if is an invalid loader", function(){
		expect(function(){
			containerLoader.setConcreteContainerLoader();
		}).toThrow();
		expect(function(){
			containerLoader.setConcreteContainerLoader(null);
		}).toThrow();
		expect(function(){
			containerLoader.setConcreteContainerLoader("loader");
		}).toThrow();
		expect(function(){
			containerLoader.setConcreteContainerLoader(123);
		}).toThrow();
		expect(function(){
			containerLoader.setConcreteContainerLoader({});
		}).toThrow();
		expect(function(){
			containerLoader.setConcreteContainerLoader([]);
		}).toThrow();
		expect(function(){
			containerLoader.setConcreteContainerLoader({ loadContainer: "loader" });
		}).toThrow();
	});

	it("Method 'setContainerDecoratorProvider': Should change the decoratorProvider if is a valid provider", function(){
		var decoratorProvider = { provide: function() {} };
		expect(function(){
			containerLoader.setContainerDecoratorProvider(decoratorProvider);
		}).not.toThrow();
	});

	it("Method 'setContainerDecoratorProvider': Should sent an exception if is an invalid provider", function(){
		expect(function(){
			containerLoader.setContainerDecoratorProvider();
		}).toThrow();
		expect(function(){
			containerLoader.setContainerDecoratorProvider(null);
		}).toThrow();
		expect(function(){
			containerLoader.setContainerDecoratorProvider("loader");
		}).toThrow();
		expect(function(){
			containerLoader.setContainerDecoratorProvider(123);
		}).toThrow();
		expect(function(){
			containerLoader.setContainerDecoratorProvider({});
		}).toThrow();
		expect(function(){
			containerLoader.setContainerDecoratorProvider([]);
		}).toThrow();
		expect(function(){
			containerLoader.setContainerDecoratorProvider({ provide: "loader" });
		}).toThrow();
	});

	it("Method 'loadContainer': ", function(done){
		containerLoader.loadContainer('login').then(function(container){
			expect(container.showFooter).toEqual(false);
			expect(container.showMenu).toEqual(true);
			expect(container.widgets[0].template).toEqual('widget/form.html');
			expect(container.widgets[0].actions[0].showAsAction).toEqual('never');
			expect(container.widgets[0].actions[0].events[0].code).toEqual('args.owner.widget.newRow();');
			expect(container.widgets[0].events[0].code).toEqual('args.owner.newRow();');
			expect(container.widgets[0].fields.length).toEqual(2);
			expect(container.widgets[0].fields[0].label).toEqual('Preference label');
			expect(container.widgets[0].fields[1].name).toEqual('NEW_FIELD');
			expect(container.widgets[0].fields[1].label).toEqual('New field');
			expect(container.widgets[0].styles[0].cssClass).toEqual('zh-inactive');
			expect(container.widgets[0].fieldGroups[0].name).toEqual('ORG changed');
			expect(container.widgets[0].fieldGroups[0].opened).toEqual(false);
			expect(container.widgets[0].widgets.length).toEqual(2);
			expect(container.widgets[0].widgets[0].id).toEqual(100);
			expect(container.widgets[0].widgets[0].name).toEqual('secondWidget');
			expect(container.widgets[0].widgets[0].label).toEqual('2');
			expect(container.widgets[0].widgets[0].fields.length).toEqual(1);
			expect(container.widgets[0].widgets[0].events.length).toEqual(1);
			expect(container.widgets[0].widgets[0].events[0].id).toEqual(102);
			expect(container.widgets[0].widgets[0].events[0].code).toEqual('args.owner.newRow();');
			expect(container.widgets[0].widgets[1].id).toEqual(1000);
			expect(container.widgets[0].widgets[1].name).toEqual('thirdWidget');
			expect(container.widgets[0].widgets[1].label).toEqual('3');
			done();
		});

	});

});