describe('CrosstabService', function () {

	var crosstabService = new CrosstabService();

	var widget;
	var emptyLabel = "EMPTY";

	beforeEach(function(){
		widget = {
			"name": "crosstabTest",
			"labelProperty": "value",
      "xAxisProperty": "dish",
      "yAxisProperty": "date",
      "emptyLabel": emptyLabel,
			"dataSource": {
				"data": [
					{
						"date": "01/05/2015",
						"dish": "Prato do dia",
						"value": "Carne cozida"
					},
					{
						"date": "02/05/2015",
						"dish": "Prato do dia",
						"value": "Frango"
					},
					{
						"date": "01/05/2015",
						"dish": "Opção A",
						"value": "Carne cozida"
					},
					{
						"date": "01/05/2015",
						"dish": "Opção B",
						"value": "Macarrão"
					},
					{
						"date": "03/05/2015",
						"dish": "Prato do dia",
						"value": "Bife boi"
					},
					{
						"date": "02/05/2015",
						"dish": "Opção A",
						"value": "Linguiça"
					},
					{
						"date": "02/05/2015",
						"dish": "Opção B",
						"value": "Macarrão"
					},
					{
						"date": "03/05/2015",
						"dish": "Opção A",
						"value": "Peixe"
					},
					{
						"date": "03/05/2015",
						"dish": "Opção B",
						"value": "Macarrão"
					},
					{
						"date": "04/05/2015",
						"dish": "Prato do dia",
						"value": "Bife porco"
					},
					{
						"date": "05/05/2015",
						"dish": "Prato do dia",
						"value": "Fígado"
					}
				]
			}
		};
	});

	it('Must throw a exception if the widget is invalid', function(){
		expect(function(){
			crosstabService.build(undefined, widget.dataSource.data);
		}).toThrow();
	});

	it('Must throw a exception if not exists the "xAxisProperty" property on widget', function(){
		delete widget.xAxisProperty;
		expect(function(){
			crosstabService.build(widget, widget.dataSource.data);
		}).toThrow();
	});

	it('Must throw a exception if not exists the "yAxisProperty" property on widget', function(){
		delete widget.yAxisProperty;
		expect(function(){
			crosstabService.build(widget, widget.dataSource.data);
		}).toThrow();
	});

	it('Must throw a exception if not exists the "labelProperty" property on widget', function(){
		delete widget.labelProperty;
		expect(function(){
			crosstabService.build(widget, widget.dataSource.data);
		}).toThrow();
	});

	it('Must fulfill a empty cell with a empty value if not exists the "emptyLabel" property on widget' , function(){
		delete widget.emptyLabel;
		var crosstabDataSet = crosstabService.build(widget, widget.dataSource.data);
		expect(crosstabDataSet[3]["Opção A"].__row).toBe(undefined);
		expect(crosstabDataSet[3]["Opção A"].label).toEqual("");
	});

	it('Must fulfill a empty cell with the "emptyLabel" widget property', function(){
		var crosstabDataSet = crosstabService.build(widget, widget.dataSource.data);
		expect(crosstabDataSet[3]["Opção A"].__row).toBe(undefined);
		expect(crosstabDataSet[3]["Opção A"].label).toEqual(emptyLabel);
	});

	it('The crosstabDataSet length must be equals to widget.crosstabColumns length', function(){
		var crosstabDataSet = crosstabService.build(widget, widget.dataSource.data);
		expect(crosstabDataSet.length).toEqual(widget.crosstabRows.length);
	});

	it('All rows of crosstabDataSet must be have all columns', function(){
		var crosstabDataSet = crosstabService.build(widget, widget.dataSource.data);
		expect(crosstabDataSet.length).toEqual(5);
		expect(crosstabDataSet.length).toEqual(widget.crosstabRows.length);

		crosstabDataSet.forEach(function(row){
			widget.crosstabColumns.forEach(function(column){
				expect(row.hasOwnProperty(column.key)).toBe(true);
			});
		});
	});

	it('The crosstabDataSet row must be have a reference to original row (__row)', function(){
		var crosstabDataSet = crosstabService.build(widget, widget.dataSource.data);
		expect(!!crosstabDataSet[0]['Prato do dia'].__row).toEqual(true);
	});

	it('The crosstabDataSet row changes must change the original row by reference', function(){
		var crosstabDataSet = crosstabService.build(widget, widget.dataSource.data);
		expect(crosstabDataSet[0]['Prato do dia'].__row).toEqual(widget.dataSource.data[0]);

		var newValue = "Value modified";
		crosstabDataSet[0]['Prato do dia'].__row.value = newValue;

		expect(widget.dataSource.data[0].value).toEqual(newValue);
		expect(crosstabDataSet[0]['Prato do dia'].__row).toEqual(widget.dataSource.data[0]);
	});
});