describe("FallbackStrategy", function() {
	describe("FallbackStrategyRetrayForEver", function() {
		var __buildPromise = function() {
			return function(name, object) {
				var defer = Q.defer();
				defer.resolve([]);
				return defer.promise;
			};
		};


		function buildMock() {
			var mockPrimary = {
				save: __buildPromise(),
				remove: __buildPromise()
			};
			var mockSeculdary = {
				save: __buildPromise(),
				remove: __buildPromise(),
			};
			for (var i in mockPrimary) {
				if (mockPrimary[i]) {
					spyOn(mockPrimary, i).and.callThrough();
					spyOn(mockSeculdary, i).and.callThrough();
				}

			}
			return {
				mockPrimary: mockPrimary,
				mockSeculdary: mockSeculdary
			};
		}
		var fallbackStrategyRetry = null;
		describe("//Save(e)", function() {
			it("Must not call no one method on primary Error", function() {
				var mooks = buildMock();
				var fallbackStrategyRetry = new FallbackStrategyRetry(mooks.mockPrimary, mooks.mockSeculdary);
				fallbackStrategyRetry.onSavePrimaryFail("Any", [],mooks.mockPrimary, mooks.mockSeculdary);
				expect(mooks.mockPrimary.save).not.toHaveBeenCalled();
				expect(mooks.mockSeculdary.save).not.toHaveBeenCalled();
			});
			it("Must not call no one method on secundary Error", function() {
				var mooks = buildMock();
				var fallbackStrategyRetry = new FallbackStrategyRetry(mooks.mockPrimary, mooks.mockSeculdary);
				fallbackStrategyRetry.onSaveSecundaryFail("Any", [],mooks.mockPrimary, mooks.mockSeculdary);
				expect(mooks.mockPrimary.save).not.toHaveBeenCalled();
				expect(mooks.mockSeculdary.save).not.toHaveBeenCalled();
			});
			it("Must re-call the secoundary save on general error", function() {
				var mooks = buildMock();
				var fallbackStrategyRetry = new FallbackStrategyRetry(mooks.mockPrimary, mooks.mockSeculdary);
				fallbackStrategyRetry.onGeneralSaveError("Any", [],mooks.mockPrimary, mooks.mockSeculdary);
				expect(mooks.mockPrimary.save).not.toHaveBeenCalled();
				expect(mooks.mockSeculdary.save).toHaveBeenCalled();
			});
		});
		describe("//Remove(e)", function() {
			it("Must not call no one method on primary Error", function() {
				var mooks = buildMock();
				var fallbackStrategyRetry = new FallbackStrategyRetry(mooks.mockPrimary, mooks.mockSeculdary);
				fallbackStrategyRetry.onRemovePrimaryFail("Any", [],mooks.mockPrimary, mooks.mockSeculdary);
				expect(mooks.mockPrimary.remove).not.toHaveBeenCalled();
				expect(mooks.mockSeculdary.remove).not.toHaveBeenCalled();
			});
			it("Must not call no one method on secundary Error", function() {
				var mooks = buildMock();
				var fallbackStrategyRetry = new FallbackStrategyRetry(mooks.mockPrimary, mooks.mockSeculdary);
				fallbackStrategyRetry.onRemoveSecundaryFail("Any", [],mooks.mockPrimary, mooks.mockSeculdary);
				expect(mooks.mockPrimary.remove).not.toHaveBeenCalled();
				expect(mooks.mockSeculdary.remove).not.toHaveBeenCalled();
			});
			it("Must re-call the secoundary remove on general error", function() {
				var mooks = buildMock();
				var fallbackStrategyRetry = new FallbackStrategyRetry(mooks.mockPrimary, mooks.mockSeculdary);
				fallbackStrategyRetry.onGeneralRemoveError("Any", [],mooks.mockPrimary, mooks.mockSeculdary);
				expect(mooks.mockPrimary.remove).not.toHaveBeenCalled();
				expect(mooks.mockSeculdary.remove).toHaveBeenCalled();
			});
		});
	});
});